# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.get_settings_response_content import GetSettingsResponseContent
from ..types.universal_login_experience_enum import UniversalLoginExperienceEnum
from ..types.update_settings_response_content import UpdateSettingsResponseContent
from .raw_client import AsyncRawPromptsClient, RawPromptsClient

if typing.TYPE_CHECKING:
    from .custom_text.client import AsyncCustomTextClient, CustomTextClient
    from .partials.client import AsyncPartialsClient, PartialsClient
    from .rendering.client import AsyncRenderingClient, RenderingClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class PromptsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawPromptsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._rendering: typing.Optional[RenderingClient] = None
        self._custom_text: typing.Optional[CustomTextClient] = None
        self._partials: typing.Optional[PartialsClient] = None

    @property
    def with_raw_response(self) -> RawPromptsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawPromptsClient
        """
        return self._raw_client

    def get_settings(self, *, request_options: typing.Optional[RequestOptions] = None) -> GetSettingsResponseContent:
        """
        Retrieve details of the Universal Login configuration of your tenant. This includes the <a href="https://auth0.com/docs/authenticate/login/auth0-universal-login/identifier-first">Identifier First Authentication</a> and <a href="https://auth0.com/docs/secure/multi-factor-authentication/fido-authentication-with-webauthn/configure-webauthn-device-biometrics-for-mfa">WebAuthn with Device Biometrics for MFA</a> features.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSettingsResponseContent
            Prompt settings successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.prompts.get_settings()
        """
        _response = self._raw_client.get_settings(request_options=request_options)
        return _response.data

    def update_settings(
        self,
        *,
        universal_login_experience: typing.Optional[UniversalLoginExperienceEnum] = OMIT,
        identifier_first: typing.Optional[bool] = OMIT,
        webauthn_platform_first_factor: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateSettingsResponseContent:
        """
        Update the Universal Login configuration of your tenant. This includes the <a href="https://auth0.com/docs/authenticate/login/auth0-universal-login/identifier-first">Identifier First Authentication</a> and <a href="https://auth0.com/docs/secure/multi-factor-authentication/fido-authentication-with-webauthn/configure-webauthn-device-biometrics-for-mfa">WebAuthn with Device Biometrics for MFA</a> features.

        Parameters
        ----------
        universal_login_experience : typing.Optional[UniversalLoginExperienceEnum]

        identifier_first : typing.Optional[bool]
            Whether identifier first is enabled or not

        webauthn_platform_first_factor : typing.Optional[bool]
            Use WebAuthn with Device Biometrics as the first authentication factor

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateSettingsResponseContent
            Prompts settings successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.prompts.update_settings()
        """
        _response = self._raw_client.update_settings(
            universal_login_experience=universal_login_experience,
            identifier_first=identifier_first,
            webauthn_platform_first_factor=webauthn_platform_first_factor,
            request_options=request_options,
        )
        return _response.data

    @property
    def rendering(self):
        if self._rendering is None:
            from .rendering.client import RenderingClient  # noqa: E402

            self._rendering = RenderingClient(client_wrapper=self._client_wrapper)
        return self._rendering

    @property
    def custom_text(self):
        if self._custom_text is None:
            from .custom_text.client import CustomTextClient  # noqa: E402

            self._custom_text = CustomTextClient(client_wrapper=self._client_wrapper)
        return self._custom_text

    @property
    def partials(self):
        if self._partials is None:
            from .partials.client import PartialsClient  # noqa: E402

            self._partials = PartialsClient(client_wrapper=self._client_wrapper)
        return self._partials


class AsyncPromptsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawPromptsClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._rendering: typing.Optional[AsyncRenderingClient] = None
        self._custom_text: typing.Optional[AsyncCustomTextClient] = None
        self._partials: typing.Optional[AsyncPartialsClient] = None

    @property
    def with_raw_response(self) -> AsyncRawPromptsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawPromptsClient
        """
        return self._raw_client

    async def get_settings(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetSettingsResponseContent:
        """
        Retrieve details of the Universal Login configuration of your tenant. This includes the <a href="https://auth0.com/docs/authenticate/login/auth0-universal-login/identifier-first">Identifier First Authentication</a> and <a href="https://auth0.com/docs/secure/multi-factor-authentication/fido-authentication-with-webauthn/configure-webauthn-device-biometrics-for-mfa">WebAuthn with Device Biometrics for MFA</a> features.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetSettingsResponseContent
            Prompt settings successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.prompts.get_settings()


        asyncio.run(main())
        """
        _response = await self._raw_client.get_settings(request_options=request_options)
        return _response.data

    async def update_settings(
        self,
        *,
        universal_login_experience: typing.Optional[UniversalLoginExperienceEnum] = OMIT,
        identifier_first: typing.Optional[bool] = OMIT,
        webauthn_platform_first_factor: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateSettingsResponseContent:
        """
        Update the Universal Login configuration of your tenant. This includes the <a href="https://auth0.com/docs/authenticate/login/auth0-universal-login/identifier-first">Identifier First Authentication</a> and <a href="https://auth0.com/docs/secure/multi-factor-authentication/fido-authentication-with-webauthn/configure-webauthn-device-biometrics-for-mfa">WebAuthn with Device Biometrics for MFA</a> features.

        Parameters
        ----------
        universal_login_experience : typing.Optional[UniversalLoginExperienceEnum]

        identifier_first : typing.Optional[bool]
            Whether identifier first is enabled or not

        webauthn_platform_first_factor : typing.Optional[bool]
            Use WebAuthn with Device Biometrics as the first authentication factor

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateSettingsResponseContent
            Prompts settings successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.prompts.update_settings()


        asyncio.run(main())
        """
        _response = await self._raw_client.update_settings(
            universal_login_experience=universal_login_experience,
            identifier_first=identifier_first,
            webauthn_platform_first_factor=webauthn_platform_first_factor,
            request_options=request_options,
        )
        return _response.data

    @property
    def rendering(self):
        if self._rendering is None:
            from .rendering.client import AsyncRenderingClient  # noqa: E402

            self._rendering = AsyncRenderingClient(client_wrapper=self._client_wrapper)
        return self._rendering

    @property
    def custom_text(self):
        if self._custom_text is None:
            from .custom_text.client import AsyncCustomTextClient  # noqa: E402

            self._custom_text = AsyncCustomTextClient(client_wrapper=self._client_wrapper)
        return self._custom_text

    @property
    def partials(self):
        if self._partials is None:
            from .partials.client import AsyncPartialsClient  # noqa: E402

            self._partials = AsyncPartialsClient(client_wrapper=self._client_wrapper)
        return self._partials



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionRequestObjectEncryptionEncValuesSupported = typing.List[str]
"""
JSON array containing a list of the JWE encryption algorithms (enc values) supported by the OP for Request Objects. These algorithms are used both when the Request Object is passed by value and when it is passed by reference.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class Rule(UniversalBaseModel):
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Name of this rule.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    ID of this rule.
    """

    enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the rule is enabled (true), or disabled (false).
    """

    script: typing.Optional[str] = pydantic.Field(default=None)
    """
    Code to be executed when this rule runs.
    """

    order: typing.Optional[float] = pydantic.Field(default=None)
    """
    Order that this rule should execute in relative to other rules. Lower-valued rules execute first.
    """

    stage: typing.Optional[str] = pydantic.Field(default=None)
    """
    Execution stage of this rule. Can be `login_success`, `login_failure`, or `pre_authorize`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

ConnectionClientIdBitbucket = str
"""
OAuth 2.0 client identifier issued by the identity provider during application registration. This value identifies your Auth0 connection to the identity provider.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamCloudEventOrgMemberRoleDeletedTypeEnum = typing.Union[
    typing.Literal["organization.member.role.deleted"], typing.Any
]



from .conftest import get_client, verify_request_count

from auth0.management import (
    CreateEventStreamWebHookRequestContent,
    EventStreamWebhookBasicAuth,
    EventStreamWebhookConfiguration,
    EventStreamWebhookDestination,
)


def test_eventStreams_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "event_streams.list_.0"
    client = get_client(test_id)
    client.event_streams.list(
        from_="from",
        take=1,
    )
    verify_request_count(test_id, "GET", "/event-streams", {"from": "from", "take": "1"}, 1)


def test_eventStreams_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "event_streams.create.0"
    client = get_client(test_id)
    client.event_streams.create(
        request=CreateEventStreamWebHookRequestContent(
            destination=EventStreamWebhookDestination(
                type="webhook",
                configuration=EventStreamWebhookConfiguration(
                    webhook_endpoint="webhook_endpoint",
                    webhook_authorization=EventStreamWebhookBasicAuth(
                        method="basic",
                        username="username",
                    ),
                ),
            ),
        ),
    )
    verify_request_count(test_id, "POST", "/event-streams", None, 1)


def test_eventStreams_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "event_streams.get.0"
    client = get_client(test_id)
    client.event_streams.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/event-streams/id", None, 1)


def test_eventStreams_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "event_streams.delete.0"
    client = get_client(test_id)
    client.event_streams.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/event-streams/id", None, 1)


def test_eventStreams_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "event_streams.update.0"
    client = get_client(test_id)
    client.event_streams.update(
        id="id",
    )
    verify_request_count(test_id, "PATCH", "/event-streams/id", None, 1)


def test_eventStreams_test() -> None:
    """Test test endpoint with WireMock"""
    test_id = "event_streams.test.0"
    client = get_client(test_id)
    client.event_streams.test(
        id="id",
        event_type="connection.created",
    )
    verify_request_count(test_id, "POST", "/event-streams/id/test", None, 1)



# This file was auto-generated by Fern from our API Definition.

ConnectionHandleLoginFromSocialGoogleApps = bool
"""
When enabled, users who sign in with their Google account through a social login will be automatically routed to this Google Workspace connection if their email domain matches the configured tenant_domain or domain_aliases. This ensures enterprise users authenticate through their organization's Google Workspace identity provider rather than through a generic Google social login, enabling access to directory-based attributes and enforcing organizational security policies. Defaults to true for new connections.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

from .event_stream_cloud_event_group_role_deleted_object_group_0 import (
    EventStreamCloudEventGroupRoleDeletedObjectGroup0,
)
from .event_stream_cloud_event_group_role_deleted_object_group_1 import (
    EventStreamCloudEventGroupRoleDeletedObjectGroup1,
)
from .event_stream_cloud_event_group_role_deleted_object_group_2 import (
    EventStreamCloudEventGroupRoleDeletedObjectGroup2,
)

EventStreamCloudEventGroupRoleDeletedObjectGroup = typing.Union[
    EventStreamCloudEventGroupRoleDeletedObjectGroup0,
    EventStreamCloudEventGroupRoleDeletedObjectGroup1,
    EventStreamCloudEventGroupRoleDeletedObjectGroup2,
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .action_module_dependency import ActionModuleDependency
from .action_module_secret import ActionModuleSecret


class CreateActionModuleVersionResponseContent(UniversalBaseModel):
    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The unique ID for this version.
    """

    module_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The ID of the parent module.
    """

    version_number: typing.Optional[int] = pydantic.Field(default=None)
    """
    The sequential version number.
    """

    code: typing.Optional[str] = pydantic.Field(default=None)
    """
    The exact source code that was published with this version.
    """

    secrets: typing.Optional[typing.List[ActionModuleSecret]] = pydantic.Field(default=None)
    """
    Secrets available to this version (name and updated_at only, values never returned).
    """

    dependencies: typing.Optional[typing.List[ActionModuleDependency]] = pydantic.Field(default=None)
    """
    Dependencies locked to this version.
    """

    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    The timestamp when this version was created.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import encode_path_param
from ..core.parse_error import ParsingError
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.list_user_blocks_by_identifier_response_content import ListUserBlocksByIdentifierResponseContent
from ..types.list_user_blocks_response_content import ListUserBlocksResponseContent
from pydantic import ValidationError


class RawUserBlocksClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_by_identifier(
        self,
        *,
        identifier: str,
        consider_brute_force_enablement: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ListUserBlocksByIdentifierResponseContent]:
        """
        Retrieve details of all [Brute-force Protection](https://auth0.com/docs/secure/attack-protection/brute-force-protection) blocks for a user with the given identifier (username, phone number, or email).

        Parameters
        ----------
        identifier : str
            Should be any of a username, phone number, or email.

        consider_brute_force_enablement : typing.Optional[bool]

                      If true and Brute Force Protection is enabled and configured to block logins, will return a list of blocked IP addresses.
                      If true and Brute Force Protection is disabled, will return an empty list.


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListUserBlocksByIdentifierResponseContent]
            User successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "user-blocks",
            method="GET",
            params={
                "identifier": identifier,
                "consider_brute_force_enablement": consider_brute_force_enablement,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListUserBlocksByIdentifierResponseContent,
                    parse_obj_as(
                        type_=ListUserBlocksByIdentifierResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete_by_identifier(
        self, *, identifier: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Remove all [Brute-force Protection](https://auth0.com/docs/secure/attack-protection/brute-force-protection) blocks for the user with the given identifier (username, phone number, or email).

        Note: This endpoint does not unblock users that were [blocked by a tenant administrator](https://auth0.com/docs/user-profile#block-and-unblock-a-user).

        Parameters
        ----------
        identifier : str
            Should be any of a username, phone number, or email.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            "user-blocks",
            method="DELETE",
            params={
                "identifier": identifier,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list(
        self,
        id: str,
        *,
        consider_brute_force_enablement: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ListUserBlocksResponseContent]:
        """
        Retrieve details of all [Brute-force Protection](https://auth0.com/docs/secure/attack-protection/brute-force-protection) blocks for the user with the given ID.

        Parameters
        ----------
        id : str
            user_id of the user blocks to retrieve.

        consider_brute_force_enablement : typing.Optional[bool]

                      If true and Brute Force Protection is enabled and configured to block logins, will return a list of blocked IP addresses.
                      If true and Brute Force Protection is disabled, will return an empty list.


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ListUserBlocksResponseContent]
            User block successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"user-blocks/{encode_path_param(id)}",
            method="GET",
            params={
                "consider_brute_force_enablement": consider_brute_force_enablement,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListUserBlocksResponseContent,
                    parse_obj_as(
                        type_=ListUserBlocksResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Remove all [Brute-force Protection](https://auth0.com/docs/secure/attack-protection/brute-force-protection) blocks for the user with the given ID.

        Note: This endpoint does not unblock users that were [blocked by a tenant administrator](https://auth0.com/docs/user-profile#block-and-unblock-a-user).

        Parameters
        ----------
        id : str
            The user_id of the user to update.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"user-blocks/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawUserBlocksClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_by_identifier(
        self,
        *,
        identifier: str,
        consider_brute_force_enablement: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ListUserBlocksByIdentifierResponseContent]:
        """
        Retrieve details of all [Brute-force Protection](https://auth0.com/docs/secure/attack-protection/brute-force-protection) blocks for a user with the given identifier (username, phone number, or email).

        Parameters
        ----------
        identifier : str
            Should be any of a username, phone number, or email.

        consider_brute_force_enablement : typing.Optional[bool]

                      If true and Brute Force Protection is enabled and configured to block logins, will return a list of blocked IP addresses.
                      If true and Brute Force Protection is disabled, will return an empty list.


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListUserBlocksByIdentifierResponseContent]
            User successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "user-blocks",
            method="GET",
            params={
                "identifier": identifier,
                "consider_brute_force_enablement": consider_brute_force_enablement,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListUserBlocksByIdentifierResponseContent,
                    parse_obj_as(
                        type_=ListUserBlocksByIdentifierResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete_by_identifier(
        self, *, identifier: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Remove all [Brute-force Protection](https://auth0.com/docs/secure/attack-protection/brute-force-protection) blocks for the user with the given identifier (username, phone number, or email).

        Note: This endpoint does not unblock users that were [blocked by a tenant administrator](https://auth0.com/docs/user-profile#block-and-unblock-a-user).

        Parameters
        ----------
        identifier : str
            Should be any of a username, phone number, or email.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "user-blocks",
            method="DELETE",
            params={
                "identifier": identifier,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list(
        self,
        id: str,
        *,
        consider_brute_force_enablement: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ListUserBlocksResponseContent]:
        """
        Retrieve details of all [Brute-force Protection](https://auth0.com/docs/secure/attack-protection/brute-force-protection) blocks for the user with the given ID.

        Parameters
        ----------
        id : str
            user_id of the user blocks to retrieve.

        consider_brute_force_enablement : typing.Optional[bool]

                      If true and Brute Force Protection is enabled and configured to block logins, will return a list of blocked IP addresses.
                      If true and Brute Force Protection is disabled, will return an empty list.


        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ListUserBlocksResponseContent]
            User block successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"user-blocks/{encode_path_param(id)}",
            method="GET",
            params={
                "consider_brute_force_enablement": consider_brute_force_enablement,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ListUserBlocksResponseContent,
                    parse_obj_as(
                        type_=ListUserBlocksResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Remove all [Brute-force Protection](https://auth0.com/docs/secure/attack-protection/brute-force-protection) blocks for the user with the given ID.

        Note: This endpoint does not unblock users that were [blocked by a tenant administrator](https://auth0.com/docs/user-profile#block-and-unblock-a-user).

        Parameters
        ----------
        id : str
            The user_id of the user to update.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"user-blocks/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from .connection_scope_item import ConnectionScopeItem

ConnectionScopeArray = typing.List[ConnectionScopeItem]
"""
Array of custom OAuth 2.0 scopes to request during authentication.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

from .user_attribute_profile_user_id import UserAttributeProfileUserId

UserAttributeProfilePatchUserId = typing.Optional[UserAttributeProfileUserId]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ConnectionsOidcMetadata(UniversalBaseModel):
    """
    Additional OIDC metadata to include in the discovery document. Only applicable when strategy=oidc, okta, or samlp.
    """

    issuer: typing.Optional[str] = pydantic.Field(default=None)
    """
    The identity provider's unique issuer identifier URL (e.g., https://accounts.google.com). Must match the 'iss' claim in ID tokens from the identity provider.
    """

    authorization_endpoint: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL of the identity provider's OAuth 2.0 authorization endpoint where users are redirected for authentication. Must be a valid HTTPS URL. This endpoint initiates the OAuth 2.0 authorization code flow.
    """

    token_endpoint: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL of the identity provider's OAuth 2.0 token endpoint where authorization codes are exchanged for access tokens. Must be a valid HTTPS URL. Required for authorization code flow but optional for implicit flow.
    """

    userinfo_endpoint: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional URL of the identity provider's UserInfo endpoint. When configured with attribute mapping, Auth0 calls this endpoint to retrieve additional user profile claims using the access token.
    """

    jwks_uri: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL of the identity provider's JSON Web Key Set (JWKS) endpoint containing public keys for signature verification. Auth0 retrieves these keys to validate ID token signatures.
    """

    registration_endpoint: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL of the OPs Dynamic Client Registration Endpoint. RECOMMENDED but not REQUIRED. https://openid.net/specs/openid-connect-discovery-1_0.html#OpenID.Registration
    """

    scopes_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of the OAuth 2.0 [RFC6749] scope values that this server supports. The server MUST support the openid scope value. Servers MAY choose not to advertise some supported scope values even when this parameter is used, although those defined in [OpenID.Core] SHOULD be listed, if supported. RECOMMENDED but not REQUIRED
    """

    response_modes_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of the OAuth 2.0 response_mode values that this OP supports. If omitted, the default for Dynamic OpenID Providers is ["query", "fragment"]
    """

    response_types_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of the OAuth 2.0 response_type values that this OP supports. Dynamic OpenID Providers MUST support the code, id_token, and the token id_token Response Type values
    """

    grant_types_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of the OAuth 2.0 Grant Type values that this OP supports. Dynamic OpenID Providers MUST support the authorization_code and implicit Grant Type values and MAY support other Grant Types. If omitted, the default value is ["authorization_code", "implicit"].
    """

    acr_values_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of the Authentication Context Class References that this OP supports
    """

    subject_types_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of the Subject Identifier types that this OP supports. Valid types include pairwise and public
    """

    id_token_signing_alg_values_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    A list of the JWS signing algorithms (alg values) supported by the OP for the ID Token to encode the Claims in a JWT. The algorithm RS256 MUST be included. The value none MAY be supported, but MUST NOT be used unless the Response Type used returns no ID Token from the Authorization Endpoint (such as when using the Authorization Code Flow). https://datatracker.ietf.org/doc/html/rfc7518
    """

    id_token_encryption_alg_values_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    JSON array containing a list of the JWE encryption algorithms (alg values) supported by the OP for the ID Token to encode the Claims in a JWT
    """

    id_token_encryption_enc_values_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    JSON array containing a list of the JWE encryption algorithms (enc values) supported by the OP for the ID Token to encode the Claims in a JWT [JWT].
    """

    userinfo_signing_alg_values_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    JSON array containing a list of the JWS [JWS] signing algorithms (alg values) [JWA] supported by the UserInfo Endpoint to encode the Claims in a JWT [JWT]. The value none MAY be included.
    """

    userinfo_encryption_alg_values_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    JSON array containing a list of the JWE [JWE] encryption algorithms (alg values) [JWA] supported by the UserInfo Endpoint to encode the Claims in a JWT [JWT].
    """

    userinfo_encryption_enc_values_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    JSON array containing a list of the JWE encryption algorithms (enc values) [JWA] supported by the UserInfo Endpoint to encode the Claims in a JWT [JWT].
    """

    request_object_signing_alg_values_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    JSON array containing a list of the JWS signing algorithms (alg values) supported by the OP for Request Objects, which are described in Section 6.1 of OpenID Connect Core 1.0 [OpenID.Core]. These algorithms are used both when the Request Object is passed by value (using the request parameter) and when it is passed by reference (using the request_uri parameter). Servers SHOULD support none and RS256.
    """

    request_object_encryption_alg_values_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    JSON array containing a list of the JWE encryption algorithms (alg values) supported by the OP for Request Objects. These algorithms are used both when the Request Object is passed by value and when it is passed by reference.
    """

    request_object_encryption_enc_values_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    JSON array containing a list of the JWE encryption algorithms (enc values) supported by the OP for Request Objects. These algorithms are used both when the Request Object is passed by value and when it is passed by reference.
    """

    token_endpoint_auth_methods_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    JSON array containing a list of Client Authentication methods supported by this Token Endpoint. The options are client_secret_post, client_secret_basic, client_secret_jwt, and private_key_jwt, as described in Section 9 of OpenID Connect Core 1.0 [OpenID.Core]. Other authentication methods MAY be defined by extensions. If omitted, the default is client_secret_basic -- the HTTP Basic Authentication Scheme specified in Section 2.3.1 of OAuth 2.0 [RFC6749].
    """

    token_endpoint_auth_signing_alg_values_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    JSON array containing a list of the JWS signing algorithms (alg values) supported by the Token Endpoint for the signature on the JWT [JWT] used to authenticate the Client at the Token Endpoint for the private_key_jwt and client_secret_jwt authentication methods. Servers SHOULD support RS256. The value none MUST NOT be used.
    """

    display_values_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    JSON array containing a list of the display parameter values that the OpenID Provider supports. These values are described in Section 3.1.2.1 of OpenID Connect Core 1.0 [OpenID.Core]
    """

    claim_types_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    JSON array containing a list of the Claim Types that the OpenID Provider supports. These Claim Types are described in Section 5.6 of OpenID Connect Core 1.0 [OpenID.Core]. If omitted, the implementation supports only normal Claims.
    """

    claims_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    JSON array containing a list of the Claim Names of the Claims that the OpenID Provider MAY be able to supply values for. Note that for privacy or other reasons, this might not be an exhaustive list.
    """

    service_documentation: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL of a page containing human-readable information that developers might want or need to know when using the OpenID Provider. In particular, if the OpenID Provider does not support Dynamic Client Registration, then information on how to register Clients needs to be provided in this documentation.
    """

    claims_locales_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Languages and scripts supported for values in Claims being returned, represented as a JSON array of BCP47 [RFC5646] language tag values. Not all languages and scripts are necessarily supported for all Claim values.
    """

    ui_locales_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    Languages and scripts supported for the user interface, represented as a JSON array of BCP47 [RFC5646] language tag values.
    """

    claims_parameter_supported: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Boolean value specifying whether the OP supports use of the claims parameter, with true indicating support. If omitted, the default value is false.
    """

    request_parameter_supported: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Boolean value specifying whether the OP supports use of the request parameter, with true indicating support. If omitted, the default value is false.
    """

    request_uri_parameter_supported: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Boolean value specifying whether the OP supports use of the request_uri parameter, with true indicating support. If omitted, the default value is false.
    """

    require_request_uri_registration: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Boolean value specifying whether the OP requires use of the request_uri parameter. If omitted, the default value is false.
    """

    op_policy_uri: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL that the OpenID Provider provides to the person registering the Client to read about the OPs requirements on how the Relying Party can use the data provided by the OP. The registration process SHOULD display this URL to the person registering the Client if it is given.
    """

    op_tos_uri: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL that the OpenID Provider provides to the person registering the Client to read about OpenID Providers terms of service. The registration process SHOULD display this URL to the person registering the Client if it is given.
    """

    end_session_endpoint: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL of the identity provider's logout/end session endpoint. When configured as a static URL, users are redirected here after logging out from Auth0. Must use HTTPS scheme.
    """

    dpop_signing_alg_values_supported: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    JSON array containing a list of the JWS signing algorithms (alg values) supported for DPoP proof JWT signing.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.parse_error import ParsingError
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...errors.bad_request_error import BadRequestError
from ...errors.forbidden_error import ForbiddenError
from ...errors.too_many_requests_error import TooManyRequestsError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.default_token_quota import DefaultTokenQuota
from ...types.get_tenant_settings_response_content import GetTenantSettingsResponseContent
from ...types.session_cookie_schema import SessionCookieSchema
from ...types.tenant_oidc_logout_settings import TenantOidcLogoutSettings
from ...types.tenant_settings_country_codes import TenantSettingsCountryCodes
from ...types.tenant_settings_device_flow import TenantSettingsDeviceFlow
from ...types.tenant_settings_dynamic_client_registration_security_mode import (
    TenantSettingsDynamicClientRegistrationSecurityMode,
)
from ...types.tenant_settings_error_page import TenantSettingsErrorPage
from ...types.tenant_settings_flags import TenantSettingsFlags
from ...types.tenant_settings_guardian_page import TenantSettingsGuardianPage
from ...types.tenant_settings_mtls import TenantSettingsMtls
from ...types.tenant_settings_nullable_security_headers import TenantSettingsNullableSecurityHeaders
from ...types.tenant_settings_password_page import TenantSettingsPasswordPage
from ...types.tenant_settings_resource_parameter_profile import TenantSettingsResourceParameterProfile
from ...types.tenant_settings_sessions import TenantSettingsSessions
from ...types.tenant_settings_supported_locales_enum import TenantSettingsSupportedLocalesEnum
from ...types.update_tenant_settings_response_content import UpdateTenantSettingsResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawSettingsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(
        self,
        *,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetTenantSettingsResponseContent]:
        """
        Retrieve tenant settings. A list of fields to include or exclude may also be specified.

        Parameters
        ----------
        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetTenantSettingsResponseContent]
            Tenant settings successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "tenants/settings",
            method="GET",
            params={
                "fields": fields,
                "include_fields": include_fields,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetTenantSettingsResponseContent,
                    parse_obj_as(
                        type_=GetTenantSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        *,
        change_password: typing.Optional[TenantSettingsPasswordPage] = OMIT,
        device_flow: typing.Optional[TenantSettingsDeviceFlow] = OMIT,
        guardian_mfa_page: typing.Optional[TenantSettingsGuardianPage] = OMIT,
        default_audience: typing.Optional[str] = OMIT,
        default_directory: typing.Optional[str] = OMIT,
        error_page: typing.Optional[TenantSettingsErrorPage] = OMIT,
        default_token_quota: typing.Optional[DefaultTokenQuota] = OMIT,
        flags: typing.Optional[TenantSettingsFlags] = OMIT,
        friendly_name: typing.Optional[str] = OMIT,
        picture_url: typing.Optional[str] = OMIT,
        support_email: typing.Optional[str] = OMIT,
        support_url: typing.Optional[str] = OMIT,
        allowed_logout_urls: typing.Optional[typing.Sequence[str]] = OMIT,
        session_lifetime: typing.Optional[int] = OMIT,
        session_lifetime_in_minutes: typing.Optional[int] = OMIT,
        idle_session_lifetime: typing.Optional[int] = OMIT,
        idle_session_lifetime_in_minutes: typing.Optional[int] = OMIT,
        ephemeral_session_lifetime: typing.Optional[int] = OMIT,
        idle_ephemeral_session_lifetime: typing.Optional[int] = OMIT,
        ephemeral_session_lifetime_in_minutes: typing.Optional[int] = OMIT,
        idle_ephemeral_session_lifetime_in_minutes: typing.Optional[int] = OMIT,
        sandbox_version: typing.Optional[str] = OMIT,
        legacy_sandbox_version: typing.Optional[str] = OMIT,
        default_redirection_uri: typing.Optional[str] = OMIT,
        enabled_locales: typing.Optional[typing.Sequence[TenantSettingsSupportedLocalesEnum]] = OMIT,
        security_headers: typing.Optional[TenantSettingsNullableSecurityHeaders] = OMIT,
        session_cookie: typing.Optional[SessionCookieSchema] = OMIT,
        sessions: typing.Optional[TenantSettingsSessions] = OMIT,
        oidc_logout: typing.Optional[TenantOidcLogoutSettings] = OMIT,
        customize_mfa_in_postlogin_action: typing.Optional[bool] = OMIT,
        allow_organization_name_in_authentication_api: typing.Optional[bool] = OMIT,
        acr_values_supported: typing.Optional[typing.Sequence[str]] = OMIT,
        mtls: typing.Optional[TenantSettingsMtls] = OMIT,
        pushed_authorization_requests_supported: typing.Optional[bool] = OMIT,
        authorization_response_iss_parameter_supported: typing.Optional[bool] = OMIT,
        skip_non_verifiable_callback_uri_confirmation_prompt: typing.Optional[bool] = OMIT,
        resource_parameter_profile: typing.Optional[TenantSettingsResourceParameterProfile] = OMIT,
        client_id_metadata_document_supported: typing.Optional[bool] = OMIT,
        enable_ai_guide: typing.Optional[bool] = OMIT,
        phone_consolidated_experience: typing.Optional[bool] = OMIT,
        include_session_metadata_in_tenant_logs: typing.Optional[bool] = OMIT,
        dynamic_client_registration_security_mode: typing.Optional[
            TenantSettingsDynamicClientRegistrationSecurityMode
        ] = OMIT,
        country_codes: typing.Optional[TenantSettingsCountryCodes] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateTenantSettingsResponseContent]:
        """
        Update settings for a tenant.

        Parameters
        ----------
        change_password : typing.Optional[TenantSettingsPasswordPage]

        device_flow : typing.Optional[TenantSettingsDeviceFlow]
            Device Flow configuration.

        guardian_mfa_page : typing.Optional[TenantSettingsGuardianPage]

        default_audience : typing.Optional[str]
            Default audience for API Authorization.

        default_directory : typing.Optional[str]
            Name of connection used for password grants at the `/token` endpoint. The following connection types are supported: LDAP, AD, Database Connections, Passwordless, Windows Azure Active Directory, ADFS.

        error_page : typing.Optional[TenantSettingsErrorPage]

        default_token_quota : typing.Optional[DefaultTokenQuota]

        flags : typing.Optional[TenantSettingsFlags]

        friendly_name : typing.Optional[str]
            Friendly name for this tenant.

        picture_url : typing.Optional[str]
            URL of logo to be shown for this tenant (recommended size: 150x150)

        support_email : typing.Optional[str]
            End-user support email.

        support_url : typing.Optional[str]
            End-user support url.

        allowed_logout_urls : typing.Optional[typing.Sequence[str]]
            URLs that are valid to redirect to after logout from Auth0.

        session_lifetime : typing.Optional[int]
            Number of hours a session will stay valid.

        session_lifetime_in_minutes : typing.Optional[int]
            Number of minutes a session will stay valid. Cannot be specified together with `session_lifetime`.

        idle_session_lifetime : typing.Optional[int]
            Number of hours for which a session can be inactive before the user must log in again.

        idle_session_lifetime_in_minutes : typing.Optional[int]
            Number of minutes a session can be inactive before the user must log in again. Cannot be specified together with `idle_session_lifetime`.

        ephemeral_session_lifetime : typing.Optional[int]
            Number of hours an ephemeral (non-persistent) session will stay valid.

        idle_ephemeral_session_lifetime : typing.Optional[int]
            Number of hours for which an ephemeral (non-persistent) session can be inactive before the user must log in again.

        ephemeral_session_lifetime_in_minutes : typing.Optional[int]
            Number of minutes an ephemeral (non-persistent) session will stay valid. Cannot be specified together with `ephemeral_session_lifetime`.

        idle_ephemeral_session_lifetime_in_minutes : typing.Optional[int]
            Number of minutes an ephemeral (non-persistent) session can be inactive before the user must log in again. Cannot be specified together with `idle_ephemeral_session_lifetime`.

        sandbox_version : typing.Optional[str]
            Selected sandbox version for the extensibility environment

        legacy_sandbox_version : typing.Optional[str]
            Selected legacy sandbox version for the extensibility environment

        default_redirection_uri : typing.Optional[str]
            The default absolute redirection uri, must be https

        enabled_locales : typing.Optional[typing.Sequence[TenantSettingsSupportedLocalesEnum]]
            Supported locales for the user interface

        security_headers : typing.Optional[TenantSettingsNullableSecurityHeaders]

        session_cookie : typing.Optional[SessionCookieSchema]

        sessions : typing.Optional[TenantSettingsSessions]

        oidc_logout : typing.Optional[TenantOidcLogoutSettings]

        customize_mfa_in_postlogin_action : typing.Optional[bool]
            Whether to enable flexible factors for MFA in the PostLogin action

        allow_organization_name_in_authentication_api : typing.Optional[bool]
            Whether to accept an organization name instead of an ID on auth endpoints

        acr_values_supported : typing.Optional[typing.Sequence[str]]
            Supported ACR values

        mtls : typing.Optional[TenantSettingsMtls]

        pushed_authorization_requests_supported : typing.Optional[bool]
            Enables the use of Pushed Authorization Requests

        authorization_response_iss_parameter_supported : typing.Optional[bool]
            Supports iss parameter in authorization responses

        skip_non_verifiable_callback_uri_confirmation_prompt : typing.Optional[bool]
            Controls whether a confirmation prompt is shown during login flows when the redirect URI uses non-verifiable callback URIs (for example, a custom URI schema such as `myapp://`, or `localhost`).
            If set to true, a confirmation prompt will not be shown. We recommend that this is set to false for improved protection from malicious apps.
            See https://auth0.com/docs/secure/security-guidance/measures-against-app-impersonation for more information.

        resource_parameter_profile : typing.Optional[TenantSettingsResourceParameterProfile]

        client_id_metadata_document_supported : typing.Optional[bool]
            Whether the authorization server supports retrieving client metadata from a client_id URL.

        enable_ai_guide : typing.Optional[bool]
            Whether Auth0 Guide (AI-powered assistance) is enabled for this tenant.

        phone_consolidated_experience : typing.Optional[bool]
            Whether Phone Consolidated Experience is enabled for this tenant.

        include_session_metadata_in_tenant_logs : typing.Optional[bool]
            Whether session metadata is included in specific tenant logs (slo, oidc_backchannel_logout_failed, oidc_backchannel_logout_succeeded).

        dynamic_client_registration_security_mode : typing.Optional[TenantSettingsDynamicClientRegistrationSecurityMode]

        country_codes : typing.Optional[TenantSettingsCountryCodes]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateTenantSettingsResponseContent]
            Tenant settings successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            "tenants/settings",
            method="PATCH",
            json={
                "change_password": convert_and_respect_annotation_metadata(
                    object_=change_password, annotation=typing.Optional[TenantSettingsPasswordPage], direction="write"
                ),
                "device_flow": convert_and_respect_annotation_metadata(
                    object_=device_flow, annotation=typing.Optional[TenantSettingsDeviceFlow], direction="write"
                ),
                "guardian_mfa_page": convert_and_respect_annotation_metadata(
                    object_=guardian_mfa_page, annotation=typing.Optional[TenantSettingsGuardianPage], direction="write"
                ),
                "default_audience": default_audience,
                "default_directory": default_directory,
                "error_page": convert_and_respect_annotation_metadata(
                    object_=error_page, annotation=typing.Optional[TenantSettingsErrorPage], direction="write"
                ),
                "default_token_quota": convert_and_respect_annotation_metadata(
                    object_=default_token_quota, annotation=typing.Optional[DefaultTokenQuota], direction="write"
                ),
                "flags": convert_and_respect_annotation_metadata(
                    object_=flags, annotation=TenantSettingsFlags, direction="write"
                ),
                "friendly_name": friendly_name,
                "picture_url": picture_url,
                "support_email": support_email,
                "support_url": support_url,
                "allowed_logout_urls": allowed_logout_urls,
                "session_lifetime": session_lifetime,
                "session_lifetime_in_minutes": session_lifetime_in_minutes,
                "idle_session_lifetime": idle_session_lifetime,
                "idle_session_lifetime_in_minutes": idle_session_lifetime_in_minutes,
                "ephemeral_session_lifetime": ephemeral_session_lifetime,
                "idle_ephemeral_session_lifetime": idle_ephemeral_session_lifetime,
                "ephemeral_session_lifetime_in_minutes": ephemeral_session_lifetime_in_minutes,
                "idle_ephemeral_session_lifetime_in_minutes": idle_ephemeral_session_lifetime_in_minutes,
                "sandbox_version": sandbox_version,
                "legacy_sandbox_version": legacy_sandbox_version,
                "default_redirection_uri": default_redirection_uri,
                "enabled_locales": enabled_locales,
                "security_headers": convert_and_respect_annotation_metadata(
                    object_=security_headers,
                    annotation=typing.Optional[TenantSettingsNullableSecurityHeaders],
                    direction="write",
                ),
                "session_cookie": convert_and_respect_annotation_metadata(
                    object_=session_cookie, annotation=typing.Optional[SessionCookieSchema], direction="write"
                ),
                "sessions": convert_and_respect_annotation_metadata(
                    object_=sessions, annotation=typing.Optional[TenantSettingsSessions], direction="write"
                ),
                "oidc_logout": convert_and_respect_annotation_metadata(
                    object_=oidc_logout, annotation=TenantOidcLogoutSettings, direction="write"
                ),
                "customize_mfa_in_postlogin_action": customize_mfa_in_postlogin_action,
                "allow_organization_name_in_authentication_api": allow_organization_name_in_authentication_api,
                "acr_values_supported": acr_values_supported,
                "mtls": convert_and_respect_annotation_metadata(
                    object_=mtls, annotation=typing.Optional[TenantSettingsMtls], direction="write"
                ),
                "pushed_authorization_requests_supported": pushed_authorization_requests_supported,
                "authorization_response_iss_parameter_supported": authorization_response_iss_parameter_supported,
                "skip_non_verifiable_callback_uri_confirmation_prompt": skip_non_verifiable_callback_uri_confirmation_prompt,
                "resource_parameter_profile": resource_parameter_profile,
                "client_id_metadata_document_supported": client_id_metadata_document_supported,
                "enable_ai_guide": enable_ai_guide,
                "phone_consolidated_experience": phone_consolidated_experience,
                "include_session_metadata_in_tenant_logs": include_session_metadata_in_tenant_logs,
                "dynamic_client_registration_security_mode": dynamic_client_registration_security_mode,
                "country_codes": convert_and_respect_annotation_metadata(
                    object_=country_codes, annotation=typing.Optional[TenantSettingsCountryCodes], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateTenantSettingsResponseContent,
                    parse_obj_as(
                        type_=UpdateTenantSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawSettingsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self,
        *,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetTenantSettingsResponseContent]:
        """
        Retrieve tenant settings. A list of fields to include or exclude may also be specified.

        Parameters
        ----------
        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetTenantSettingsResponseContent]
            Tenant settings successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "tenants/settings",
            method="GET",
            params={
                "fields": fields,
                "include_fields": include_fields,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetTenantSettingsResponseContent,
                    parse_obj_as(
                        type_=GetTenantSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        *,
        change_password: typing.Optional[TenantSettingsPasswordPage] = OMIT,
        device_flow: typing.Optional[TenantSettingsDeviceFlow] = OMIT,
        guardian_mfa_page: typing.Optional[TenantSettingsGuardianPage] = OMIT,
        default_audience: typing.Optional[str] = OMIT,
        default_directory: typing.Optional[str] = OMIT,
        error_page: typing.Optional[TenantSettingsErrorPage] = OMIT,
        default_token_quota: typing.Optional[DefaultTokenQuota] = OMIT,
        flags: typing.Optional[TenantSettingsFlags] = OMIT,
        friendly_name: typing.Optional[str] = OMIT,
        picture_url: typing.Optional[str] = OMIT,
        support_email: typing.Optional[str] = OMIT,
        support_url: typing.Optional[str] = OMIT,
        allowed_logout_urls: typing.Optional[typing.Sequence[str]] = OMIT,
        session_lifetime: typing.Optional[int] = OMIT,
        session_lifetime_in_minutes: typing.Optional[int] = OMIT,
        idle_session_lifetime: typing.Optional[int] = OMIT,
        idle_session_lifetime_in_minutes: typing.Optional[int] = OMIT,
        ephemeral_session_lifetime: typing.Optional[int] = OMIT,
        idle_ephemeral_session_lifetime: typing.Optional[int] = OMIT,
        ephemeral_session_lifetime_in_minutes: typing.Optional[int] = OMIT,
        idle_ephemeral_session_lifetime_in_minutes: typing.Optional[int] = OMIT,
        sandbox_version: typing.Optional[str] = OMIT,
        legacy_sandbox_version: typing.Optional[str] = OMIT,
        default_redirection_uri: typing.Optional[str] = OMIT,
        enabled_locales: typing.Optional[typing.Sequence[TenantSettingsSupportedLocalesEnum]] = OMIT,
        security_headers: typing.Optional[TenantSettingsNullableSecurityHeaders] = OMIT,
        session_cookie: typing.Optional[SessionCookieSchema] = OMIT,
        sessions: typing.Optional[TenantSettingsSessions] = OMIT,
        oidc_logout: typing.Optional[TenantOidcLogoutSettings] = OMIT,
        customize_mfa_in_postlogin_action: typing.Optional[bool] = OMIT,
        allow_organization_name_in_authentication_api: typing.Optional[bool] = OMIT,
        acr_values_supported: typing.Optional[typing.Sequence[str]] = OMIT,
        mtls: typing.Optional[TenantSettingsMtls] = OMIT,
        pushed_authorization_requests_supported: typing.Optional[bool] = OMIT,
        authorization_response_iss_parameter_supported: typing.Optional[bool] = OMIT,
        skip_non_verifiable_callback_uri_confirmation_prompt: typing.Optional[bool] = OMIT,
        resource_parameter_profile: typing.Optional[TenantSettingsResourceParameterProfile] = OMIT,
        client_id_metadata_document_supported: typing.Optional[bool] = OMIT,
        enable_ai_guide: typing.Optional[bool] = OMIT,
        phone_consolidated_experience: typing.Optional[bool] = OMIT,
        include_session_metadata_in_tenant_logs: typing.Optional[bool] = OMIT,
        dynamic_client_registration_security_mode: typing.Optional[
            TenantSettingsDynamicClientRegistrationSecurityMode
        ] = OMIT,
        country_codes: typing.Optional[TenantSettingsCountryCodes] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateTenantSettingsResponseContent]:
        """
        Update settings for a tenant.

        Parameters
        ----------
        change_password : typing.Optional[TenantSettingsPasswordPage]

        device_flow : typing.Optional[TenantSettingsDeviceFlow]
            Device Flow configuration.

        guardian_mfa_page : typing.Optional[TenantSettingsGuardianPage]

        default_audience : typing.Optional[str]
            Default audience for API Authorization.

        default_directory : typing.Optional[str]
            Name of connection used for password grants at the `/token` endpoint. The following connection types are supported: LDAP, AD, Database Connections, Passwordless, Windows Azure Active Directory, ADFS.

        error_page : typing.Optional[TenantSettingsErrorPage]

        default_token_quota : typing.Optional[DefaultTokenQuota]

        flags : typing.Optional[TenantSettingsFlags]

        friendly_name : typing.Optional[str]
            Friendly name for this tenant.

        picture_url : typing.Optional[str]
            URL of logo to be shown for this tenant (recommended size: 150x150)

        support_email : typing.Optional[str]
            End-user support email.

        support_url : typing.Optional[str]
            End-user support url.

        allowed_logout_urls : typing.Optional[typing.Sequence[str]]
            URLs that are valid to redirect to after logout from Auth0.

        session_lifetime : typing.Optional[int]
            Number of hours a session will stay valid.

        session_lifetime_in_minutes : typing.Optional[int]
            Number of minutes a session will stay valid. Cannot be specified together with `session_lifetime`.

        idle_session_lifetime : typing.Optional[int]
            Number of hours for which a session can be inactive before the user must log in again.

        idle_session_lifetime_in_minutes : typing.Optional[int]
            Number of minutes a session can be inactive before the user must log in again. Cannot be specified together with `idle_session_lifetime`.

        ephemeral_session_lifetime : typing.Optional[int]
            Number of hours an ephemeral (non-persistent) session will stay valid.

        idle_ephemeral_session_lifetime : typing.Optional[int]
            Number of hours for which an ephemeral (non-persistent) session can be inactive before the user must log in again.

        ephemeral_session_lifetime_in_minutes : typing.Optional[int]
            Number of minutes an ephemeral (non-persistent) session will stay valid. Cannot be specified together with `ephemeral_session_lifetime`.

        idle_ephemeral_session_lifetime_in_minutes : typing.Optional[int]
            Number of minutes an ephemeral (non-persistent) session can be inactive before the user must log in again. Cannot be specified together with `idle_ephemeral_session_lifetime`.

        sandbox_version : typing.Optional[str]
            Selected sandbox version for the extensibility environment

        legacy_sandbox_version : typing.Optional[str]
            Selected legacy sandbox version for the extensibility environment

        default_redirection_uri : typing.Optional[str]
            The default absolute redirection uri, must be https

        enabled_locales : typing.Optional[typing.Sequence[TenantSettingsSupportedLocalesEnum]]
            Supported locales for the user interface

        security_headers : typing.Optional[TenantSettingsNullableSecurityHeaders]

        session_cookie : typing.Optional[SessionCookieSchema]

        sessions : typing.Optional[TenantSettingsSessions]

        oidc_logout : typing.Optional[TenantOidcLogoutSettings]

        customize_mfa_in_postlogin_action : typing.Optional[bool]
            Whether to enable flexible factors for MFA in the PostLogin action

        allow_organization_name_in_authentication_api : typing.Optional[bool]
            Whether to accept an organization name instead of an ID on auth endpoints

        acr_values_supported : typing.Optional[typing.Sequence[str]]
            Supported ACR values

        mtls : typing.Optional[TenantSettingsMtls]

        pushed_authorization_requests_supported : typing.Optional[bool]
            Enables the use of Pushed Authorization Requests

        authorization_response_iss_parameter_supported : typing.Optional[bool]
            Supports iss parameter in authorization responses

        skip_non_verifiable_callback_uri_confirmation_prompt : typing.Optional[bool]
            Controls whether a confirmation prompt is shown during login flows when the redirect URI uses non-verifiable callback URIs (for example, a custom URI schema such as `myapp://`, or `localhost`).
            If set to true, a confirmation prompt will not be shown. We recommend that this is set to false for improved protection from malicious apps.
            See https://auth0.com/docs/secure/security-guidance/measures-against-app-impersonation for more information.

        resource_parameter_profile : typing.Optional[TenantSettingsResourceParameterProfile]

        client_id_metadata_document_supported : typing.Optional[bool]
            Whether the authorization server supports retrieving client metadata from a client_id URL.

        enable_ai_guide : typing.Optional[bool]
            Whether Auth0 Guide (AI-powered assistance) is enabled for this tenant.

        phone_consolidated_experience : typing.Optional[bool]
            Whether Phone Consolidated Experience is enabled for this tenant.

        include_session_metadata_in_tenant_logs : typing.Optional[bool]
            Whether session metadata is included in specific tenant logs (slo, oidc_backchannel_logout_failed, oidc_backchannel_logout_succeeded).

        dynamic_client_registration_security_mode : typing.Optional[TenantSettingsDynamicClientRegistrationSecurityMode]

        country_codes : typing.Optional[TenantSettingsCountryCodes]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateTenantSettingsResponseContent]
            Tenant settings successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "tenants/settings",
            method="PATCH",
            json={
                "change_password": convert_and_respect_annotation_metadata(
                    object_=change_password, annotation=typing.Optional[TenantSettingsPasswordPage], direction="write"
                ),
                "device_flow": convert_and_respect_annotation_metadata(
                    object_=device_flow, annotation=typing.Optional[TenantSettingsDeviceFlow], direction="write"
                ),
                "guardian_mfa_page": convert_and_respect_annotation_metadata(
                    object_=guardian_mfa_page, annotation=typing.Optional[TenantSettingsGuardianPage], direction="write"
                ),
                "default_audience": default_audience,
                "default_directory": default_directory,
                "error_page": convert_and_respect_annotation_metadata(
                    object_=error_page, annotation=typing.Optional[TenantSettingsErrorPage], direction="write"
                ),
                "default_token_quota": convert_and_respect_annotation_metadata(
                    object_=default_token_quota, annotation=typing.Optional[DefaultTokenQuota], direction="write"
                ),
                "flags": convert_and_respect_annotation_metadata(
                    object_=flags, annotation=TenantSettingsFlags, direction="write"
                ),
                "friendly_name": friendly_name,
                "picture_url": picture_url,
                "support_email": support_email,
                "support_url": support_url,
                "allowed_logout_urls": allowed_logout_urls,
                "session_lifetime": session_lifetime,
                "session_lifetime_in_minutes": session_lifetime_in_minutes,
                "idle_session_lifetime": idle_session_lifetime,
                "idle_session_lifetime_in_minutes": idle_session_lifetime_in_minutes,
                "ephemeral_session_lifetime": ephemeral_session_lifetime,
                "idle_ephemeral_session_lifetime": idle_ephemeral_session_lifetime,
                "ephemeral_session_lifetime_in_minutes": ephemeral_session_lifetime_in_minutes,
                "idle_ephemeral_session_lifetime_in_minutes": idle_ephemeral_session_lifetime_in_minutes,
                "sandbox_version": sandbox_version,
                "legacy_sandbox_version": legacy_sandbox_version,
                "default_redirection_uri": default_redirection_uri,
                "enabled_locales": enabled_locales,
                "security_headers": convert_and_respect_annotation_metadata(
                    object_=security_headers,
                    annotation=typing.Optional[TenantSettingsNullableSecurityHeaders],
                    direction="write",
                ),
                "session_cookie": convert_and_respect_annotation_metadata(
                    object_=session_cookie, annotation=typing.Optional[SessionCookieSchema], direction="write"
                ),
                "sessions": convert_and_respect_annotation_metadata(
                    object_=sessions, annotation=typing.Optional[TenantSettingsSessions], direction="write"
                ),
                "oidc_logout": convert_and_respect_annotation_metadata(
                    object_=oidc_logout, annotation=TenantOidcLogoutSettings, direction="write"
                ),
                "customize_mfa_in_postlogin_action": customize_mfa_in_postlogin_action,
                "allow_organization_name_in_authentication_api": allow_organization_name_in_authentication_api,
                "acr_values_supported": acr_values_supported,
                "mtls": convert_and_respect_annotation_metadata(
                    object_=mtls, annotation=typing.Optional[TenantSettingsMtls], direction="write"
                ),
                "pushed_authorization_requests_supported": pushed_authorization_requests_supported,
                "authorization_response_iss_parameter_supported": authorization_response_iss_parameter_supported,
                "skip_non_verifiable_callback_uri_confirmation_prompt": skip_non_verifiable_callback_uri_confirmation_prompt,
                "resource_parameter_profile": resource_parameter_profile,
                "client_id_metadata_document_supported": client_id_metadata_document_supported,
                "enable_ai_guide": enable_ai_guide,
                "phone_consolidated_experience": phone_consolidated_experience,
                "include_session_metadata_in_tenant_logs": include_session_metadata_in_tenant_logs,
                "dynamic_client_registration_security_mode": dynamic_client_registration_security_mode,
                "country_codes": convert_and_respect_annotation_metadata(
                    object_=country_codes, annotation=typing.Optional[TenantSettingsCountryCodes], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateTenantSettingsResponseContent,
                    parse_obj_as(
                        type_=UpdateTenantSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

UserMetadata = typing.Dict[str, typing.Any]
"""
Data related to the user that does not affect the application's core functionality.
"""



from __future__ import annotations

import base64
import re
from json import dumps
from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Union

from .core.request_options import RequestOptions

import httpx
from .client import AsyncAuth0, Auth0
from .token_provider import TokenProvider

CUSTOM_DOMAIN_HEADER = "Auth0-Custom-Domain"

WHITELISTED_PATH_PATTERNS: List[re.Pattern[str]] = [
    re.compile(r"^/api/v2/jobs/verification-email$"),
    re.compile(r"^/api/v2/tickets/email-verification$"),
    re.compile(r"^/api/v2/tickets/password-change$"),
    re.compile(r"^/api/v2/organizations/[^/]+/invitations$"),
    re.compile(r"^/api/v2/users$"),
    re.compile(r"^/api/v2/users/[^/]+$"),
    re.compile(r"^/api/v2/guardian/enrollments/ticket$"),
    re.compile(r"^/api/v2/self-service-profiles/[^/]+/sso-ticket$"),
]


def _is_path_whitelisted(path: str) -> bool:
    """Check if the given path is whitelisted for the custom domain header."""
    return any(p.match(path) for p in WHITELISTED_PATH_PATTERNS)


def _enforce_custom_domain_whitelist(request: httpx.Request) -> None:
    """httpx event hook that strips Auth0-Custom-Domain on non-whitelisted paths."""
    if CUSTOM_DOMAIN_HEADER in request.headers and not _is_path_whitelisted(
        request.url.path
    ):
        del request.headers[CUSTOM_DOMAIN_HEADER]


def CustomDomainHeader(domain: str) -> RequestOptions:
    """Create request options that set the Auth0-Custom-Domain header for a single request.

    When both a global custom_domain (set at client init) and a per-request
    custom_domain_header are provided, the per-request value takes precedence.
    The header is only sent on whitelisted endpoints.

    Usage::

        from auth0.management import ManagementClient, CustomDomainHeader

        client = ManagementClient(domain="tenant.auth0.com", token="TOKEN")
        client.users.create(
            connection="Username-Password-Authentication",
            email="user@example.com",
            password="...",
            request_options=CustomDomainHeader("login.mycompany.com"),
        )
    """
    return {"additional_headers": {CUSTOM_DOMAIN_HEADER: domain}}

if TYPE_CHECKING:
    from .actions.client import ActionsClient, AsyncActionsClient
    from .anomaly.client import AnomalyClient, AsyncAnomalyClient
    from .attack_protection.client import AsyncAttackProtectionClient, AttackProtectionClient
    from .branding.client import AsyncBrandingClient, BrandingClient
    from .client_grants.client import AsyncClientGrantsClient, ClientGrantsClient
    from .clients.client import AsyncClientsClient, ClientsClient
    from .connections.client import AsyncConnectionsClient, ConnectionsClient
    from .custom_domains.client import AsyncCustomDomainsClient, CustomDomainsClient
    from .device_credentials.client import AsyncDeviceCredentialsClient, DeviceCredentialsClient
    from .email_templates.client import AsyncEmailTemplatesClient, EmailTemplatesClient
    from .emails.client import AsyncEmailsClient, EmailsClient
    from .event_streams.client import AsyncEventStreamsClient, EventStreamsClient
    from .flows.client import AsyncFlowsClient, FlowsClient
    from .forms.client import AsyncFormsClient, FormsClient
    from .guardian.client import AsyncGuardianClient, GuardianClient
    from .hooks.client import AsyncHooksClient, HooksClient
    from .jobs.client import AsyncJobsClient, JobsClient
    from .keys.client import AsyncKeysClient, KeysClient
    from .log_streams.client import AsyncLogStreamsClient, LogStreamsClient
    from .logs.client import AsyncLogsClient, LogsClient
    from .network_acls.client import AsyncNetworkAclsClient, NetworkAclsClient
    from .organizations.client import AsyncOrganizationsClient, OrganizationsClient
    from .prompts.client import AsyncPromptsClient, PromptsClient
    from .refresh_tokens.client import AsyncRefreshTokensClient, RefreshTokensClient
    from .resource_servers.client import AsyncResourceServersClient, ResourceServersClient
    from .roles.client import AsyncRolesClient, RolesClient
    from .rules.client import AsyncRulesClient, RulesClient
    from .rules_configs.client import AsyncRulesConfigsClient, RulesConfigsClient
    from .self_service_profiles.client import AsyncSelfServiceProfilesClient, SelfServiceProfilesClient
    from .sessions.client import AsyncSessionsClient, SessionsClient
    from .stats.client import AsyncStatsClient, StatsClient
    from .supplemental_signals.client import AsyncSupplementalSignalsClient, SupplementalSignalsClient
    from .tenants.client import AsyncTenantsClient, TenantsClient
    from .tickets.client import AsyncTicketsClient, TicketsClient
    from .token_exchange_profiles.client import AsyncTokenExchangeProfilesClient, TokenExchangeProfilesClient
    from .user_blocks.client import AsyncUserBlocksClient, UserBlocksClient
    from .user_grants.client import AsyncUserGrantsClient, UserGrantsClient
    from .users.client import AsyncUsersClient, UsersClient
    from .verifiable_credentials.client import AsyncVerifiableCredentialsClient, VerifiableCredentialsClient


class ManagementClient:
    """
    Auth0 Management API client with automatic token management.

    Supports two initialization patterns:

    1. With an existing token:

        client = ManagementClient(
            domain="tenant.auth0.com",
            token="your_token"
        )

    2. With client credentials (automatic token acquisition and refresh):

        client = ManagementClient(
            domain="tenant.auth0.com",
            client_id="your_client_id",
            client_secret="your_client_secret"
        )

    Parameters
    ----------
    domain : str
        Your Auth0 domain (e.g., "your-tenant.auth0.com").
    token : Optional[Union[str, Callable[[], str]]]
        A static token string or a callable that returns a token.
        Required if client_id/client_secret are not provided.
    client_id : Optional[str]
        Your Auth0 application client ID.
        Required along with client_secret for automatic token acquisition.
    client_secret : Optional[str]
        Your Auth0 application client secret.
        Required along with client_id for automatic token acquisition.
    audience : Optional[str]
        The API audience. Defaults to https://{domain}/api/v2/
    headers : Optional[Dict[str, str]]
        Additional headers to send with requests.
    custom_domain : Optional[str]
        A custom domain to send via the Auth0-Custom-Domain header.
        The header is only sent on whitelisted endpoints. Use
        ``CustomDomainHeader()`` for per-request overrides.
    client_info : Optional[Dict[str, Any]]
        Custom telemetry data for the Auth0-Client header. When provided,
        overrides the default SDK telemetry. Useful for wrapper SDKs that
        need to identify themselves (e.g., ``{"name": "my-sdk", "version": "1.0.0"}``).
    timeout : Optional[float]
        Request timeout in seconds. Defaults to 60.
    httpx_client : Optional[httpx.Client]
        Custom httpx client for requests.

    Raises
    ------
    ValueError
        If neither token nor client_id/client_secret are provided.
    """

    def __init__(
        self,
        *,
        domain: str,
        token: Optional[Union[str, Callable[[], str]]] = None,
        client_id: Optional[str] = None,
        client_secret: Optional[str] = None,
        audience: Optional[str] = None,
        headers: Optional[Dict[str, str]] = None,
        custom_domain: Optional[str] = None,
        client_info: Optional[Dict[str, Any]] = None,
        timeout: Optional[float] = None,
        httpx_client: Optional[httpx.Client] = None,
    ):
        # Validate auth options
        has_token = token is not None
        has_credentials = client_id is not None and client_secret is not None

        if not has_token and not has_credentials:
            raise ValueError("Either 'token' or both 'client_id' and 'client_secret' must be provided")

        # Create token supplier
        if has_credentials and not has_token:
            provider = TokenProvider(
                domain=domain,
                client_id=client_id,  # type: ignore[arg-type]
                client_secret=client_secret,  # type: ignore[arg-type]
                audience=audience,
            )
            resolved_token: Union[str, Callable[[], str]] = provider.get_token
        else:
            resolved_token = token  # type: ignore[assignment]

        # Set up custom domain header with whitelist enforcement
        if custom_domain is not None:
            headers = {**(headers or {}), CUSTOM_DOMAIN_HEADER: custom_domain}
            if httpx_client is None:
                httpx_client = httpx.Client(timeout=timeout or 60, follow_redirects=True)
            httpx_client.event_hooks.setdefault("request", []).append(
                _enforce_custom_domain_whitelist
            )

        # Encode client_info into Auth0-Client header to override default telemetry
        if client_info is not None:
            encoded = base64.b64encode(
                dumps(client_info).encode("utf-8")
            ).decode()
            headers = {**(headers or {}), "Auth0-Client": encoded}

        # Create underlying client
        self._api = Auth0(
            base_url=f"https://{domain}/api/v2",
            token=resolved_token,
            headers=headers,
            timeout=timeout,
            httpx_client=httpx_client,
        )

    # Forward all sub-client properties
    @property
    def actions(self) -> "ActionsClient":
        return self._api.actions

    @property
    def anomaly(self) -> "AnomalyClient":
        return self._api.anomaly

    @property
    def attack_protection(self) -> "AttackProtectionClient":
        return self._api.attack_protection

    @property
    def branding(self) -> "BrandingClient":
        return self._api.branding

    @property
    def client_grants(self) -> "ClientGrantsClient":
        return self._api.client_grants

    @property
    def clients(self) -> "ClientsClient":
        return self._api.clients

    @property
    def connections(self) -> "ConnectionsClient":
        return self._api.connections

    @property
    def custom_domains(self) -> "CustomDomainsClient":
        return self._api.custom_domains

    @property
    def device_credentials(self) -> "DeviceCredentialsClient":
        return self._api.device_credentials

    @property
    def email_templates(self) -> "EmailTemplatesClient":
        return self._api.email_templates

    @property
    def emails(self) -> "EmailsClient":
        return self._api.emails

    @property
    def event_streams(self) -> "EventStreamsClient":
        return self._api.event_streams

    @property
    def flows(self) -> "FlowsClient":
        return self._api.flows

    @property
    def forms(self) -> "FormsClient":
        return self._api.forms

    @property
    def guardian(self) -> "GuardianClient":
        return self._api.guardian

    @property
    def hooks(self) -> "HooksClient":
        return self._api.hooks

    @property
    def jobs(self) -> "JobsClient":
        return self._api.jobs

    @property
    def keys(self) -> "KeysClient":
        return self._api.keys

    @property
    def log_streams(self) -> "LogStreamsClient":
        return self._api.log_streams

    @property
    def logs(self) -> "LogsClient":
        return self._api.logs

    @property
    def network_acls(self) -> "NetworkAclsClient":
        return self._api.network_acls

    @property
    def organizations(self) -> "OrganizationsClient":
        return self._api.organizations

    @property
    def prompts(self) -> "PromptsClient":
        return self._api.prompts

    @property
    def refresh_tokens(self) -> "RefreshTokensClient":
        return self._api.refresh_tokens

    @property
    def resource_servers(self) -> "ResourceServersClient":
        return self._api.resource_servers

    @property
    def roles(self) -> "RolesClient":
        return self._api.roles

    @property
    def rules(self) -> "RulesClient":
        return self._api.rules

    @property
    def rules_configs(self) -> "RulesConfigsClient":
        return self._api.rules_configs

    @property
    def self_service_profiles(self) -> "SelfServiceProfilesClient":
        return self._api.self_service_profiles

    @property
    def sessions(self) -> "SessionsClient":
        return self._api.sessions

    @property
    def stats(self) -> "StatsClient":
        return self._api.stats

    @property
    def supplemental_signals(self) -> "SupplementalSignalsClient":
        return self._api.supplemental_signals

    @property
    def tenants(self) -> "TenantsClient":
        return self._api.tenants

    @property
    def tickets(self) -> "TicketsClient":
        return self._api.tickets

    @property
    def token_exchange_profiles(self) -> "TokenExchangeProfilesClient":
        return self._api.token_exchange_profiles

    @property
    def user_blocks(self) -> "UserBlocksClient":
        return self._api.user_blocks

    @property
    def user_grants(self) -> "UserGrantsClient":
        return self._api.user_grants

    @property
    def users(self) -> "UsersClient":
        return self._api.users

    @property
    def verifiable_credentials(self) -> "VerifiableCredentialsClient":
        return self._api.verifiable_credentials


class AsyncManagementClient:
    """
    Async Auth0 Management API client with automatic token management.

    Supports two initialization patterns:

    1. With an existing token:

        client = AsyncManagementClient(
            domain="tenant.auth0.com",
            token="your_token"
        )

    2. With client credentials (automatic token acquisition and refresh):

        client = AsyncManagementClient(
            domain="tenant.auth0.com",
            client_id="your_client_id",
            client_secret="your_client_secret"
        )

    Parameters
    ----------
    domain : str
        Your Auth0 domain (e.g., "your-tenant.auth0.com").
    token : Optional[Union[str, Callable[[], str]]]
        A static token string or a callable that returns a token.
        Required if client_id/client_secret are not provided.
    client_id : Optional[str]
        Your Auth0 application client ID.
        Required along with client_secret for automatic token acquisition.
    client_secret : Optional[str]
        Your Auth0 application client secret.
        Required along with client_id for automatic token acquisition.
    audience : Optional[str]
        The API audience. Defaults to https://{domain}/api/v2/
    headers : Optional[Dict[str, str]]
        Additional headers to send with requests.
    custom_domain : Optional[str]
        A custom domain to send via the Auth0-Custom-Domain header.
        The header is only sent on whitelisted endpoints. Use
        ``CustomDomainHeader()`` for per-request overrides.
    client_info : Optional[Dict[str, Any]]
        Custom telemetry data for the Auth0-Client header. When provided,
        overrides the default SDK telemetry. Useful for wrapper SDKs that
        need to identify themselves (e.g., ``{"name": "my-sdk", "version": "1.0.0"}``).
    timeout : Optional[float]
        Request timeout in seconds. Defaults to 60.
    httpx_client : Optional[httpx.AsyncClient]
        Custom httpx async client for requests.

    Raises
    ------
    ValueError
        If neither token nor client_id/client_secret are provided.
    """

    def __init__(
        self,
        *,
        domain: str,
        token: Optional[Union[str, Callable[[], str]]] = None,
        client_id: Optional[str] = None,
        client_secret: Optional[str] = None,
        audience: Optional[str] = None,
        headers: Optional[Dict[str, str]] = None,
        custom_domain: Optional[str] = None,
        client_info: Optional[Dict[str, Any]] = None,
        timeout: Optional[float] = None,
        httpx_client: Optional[httpx.AsyncClient] = None,
    ):
        # Validate auth options
        has_token = token is not None
        has_credentials = client_id is not None and client_secret is not None

        if not has_token and not has_credentials:
            raise ValueError("Either 'token' or both 'client_id' and 'client_secret' must be provided")

        # Create token supplier
        # Note: AsyncAuth0Api expects a sync callable for token, so we use
        # the sync TokenProvider. This is safe because httpx sync calls work
        # in async contexts.
        if has_credentials and not has_token:
            provider = TokenProvider(
                domain=domain,
                client_id=client_id,  # type: ignore[arg-type]
                client_secret=client_secret,  # type: ignore[arg-type]
                audience=audience,
            )
            resolved_token: Union[str, Callable[[], str]] = provider.get_token
        else:
            resolved_token = token  # type: ignore[assignment]

        # Set up custom domain header with whitelist enforcement
        if custom_domain is not None:
            headers = {**(headers or {}), CUSTOM_DOMAIN_HEADER: custom_domain}
            if httpx_client is None:
                httpx_client = httpx.AsyncClient(timeout=timeout or 60, follow_redirects=True)
            httpx_client.event_hooks.setdefault("request", []).append(
                _enforce_custom_domain_whitelist
            )

        # Encode client_info into Auth0-Client header to override default telemetry
        if client_info is not None:
            encoded = base64.b64encode(
                dumps(client_info).encode("utf-8")
            ).decode()
            headers = {**(headers or {}), "Auth0-Client": encoded}

        # Create underlying client
        self._api = AsyncAuth0(
            base_url=f"https://{domain}/api/v2",
            token=resolved_token,
            headers=headers,
            timeout=timeout,
            httpx_client=httpx_client,
        )

    # Forward all sub-client properties
    @property
    def actions(self) -> "AsyncActionsClient":
        return self._api.actions

    @property
    def anomaly(self) -> "AsyncAnomalyClient":
        return self._api.anomaly

    @property
    def attack_protection(self) -> "AsyncAttackProtectionClient":
        return self._api.attack_protection

    @property
    def branding(self) -> "AsyncBrandingClient":
        return self._api.branding

    @property
    def client_grants(self) -> "AsyncClientGrantsClient":
        return self._api.client_grants

    @property
    def clients(self) -> "AsyncClientsClient":
        return self._api.clients

    @property
    def connections(self) -> "AsyncConnectionsClient":
        return self._api.connections

    @property
    def custom_domains(self) -> "AsyncCustomDomainsClient":
        return self._api.custom_domains

    @property
    def device_credentials(self) -> "AsyncDeviceCredentialsClient":
        return self._api.device_credentials

    @property
    def email_templates(self) -> "AsyncEmailTemplatesClient":
        return self._api.email_templates

    @property
    def emails(self) -> "AsyncEmailsClient":
        return self._api.emails

    @property
    def event_streams(self) -> "AsyncEventStreamsClient":
        return self._api.event_streams

    @property
    def flows(self) -> "AsyncFlowsClient":
        return self._api.flows

    @property
    def forms(self) -> "AsyncFormsClient":
        return self._api.forms

    @property
    def guardian(self) -> "AsyncGuardianClient":
        return self._api.guardian

    @property
    def hooks(self) -> "AsyncHooksClient":
        return self._api.hooks

    @property
    def jobs(self) -> "AsyncJobsClient":
        return self._api.jobs

    @property
    def keys(self) -> "AsyncKeysClient":
        return self._api.keys

    @property
    def log_streams(self) -> "AsyncLogStreamsClient":
        return self._api.log_streams

    @property
    def logs(self) -> "AsyncLogsClient":
        return self._api.logs

    @property
    def network_acls(self) -> "AsyncNetworkAclsClient":
        return self._api.network_acls

    @property
    def organizations(self) -> "AsyncOrganizationsClient":
        return self._api.organizations

    @property
    def prompts(self) -> "AsyncPromptsClient":
        return self._api.prompts

    @property
    def refresh_tokens(self) -> "AsyncRefreshTokensClient":
        return self._api.refresh_tokens

    @property
    def resource_servers(self) -> "AsyncResourceServersClient":
        return self._api.resource_servers

    @property
    def roles(self) -> "AsyncRolesClient":
        return self._api.roles

    @property
    def rules(self) -> "AsyncRulesClient":
        return self._api.rules

    @property
    def rules_configs(self) -> "AsyncRulesConfigsClient":
        return self._api.rules_configs

    @property
    def self_service_profiles(self) -> "AsyncSelfServiceProfilesClient":
        return self._api.self_service_profiles

    @property
    def sessions(self) -> "AsyncSessionsClient":
        return self._api.sessions

    @property
    def stats(self) -> "AsyncStatsClient":
        return self._api.stats

    @property
    def supplemental_signals(self) -> "AsyncSupplementalSignalsClient":
        return self._api.supplemental_signals

    @property
    def tenants(self) -> "AsyncTenantsClient":
        return self._api.tenants

    @property
    def tickets(self) -> "AsyncTicketsClient":
        return self._api.tickets

    @property
    def token_exchange_profiles(self) -> "AsyncTokenExchangeProfilesClient":
        return self._api.token_exchange_profiles

    @property
    def user_blocks(self) -> "AsyncUserBlocksClient":
        return self._api.user_blocks

    @property
    def user_grants(self) -> "AsyncUserGrantsClient":
        return self._api.user_grants

    @property
    def users(self) -> "AsyncUsersClient":
        return self._api.users

    @property
    def verifiable_credentials(self) -> "AsyncVerifiableCredentialsClient":
        return self._api.verifiable_credentials



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionIdTokenSigningAlgValuesSupported = typing.List[str]
"""
A list of the JWS signing algorithms (alg values) supported by the OP for the ID Token to encode the Claims in a JWT. The algorithm RS256 MUST be included. The value none MAY be supported, but MUST NOT be used unless the Response Type used returns no ID Token from the Authorization Endpoint (such as when using the Authorization Code Flow). https://datatracker.ietf.org/doc/html/rfc7518
"""



# This file was auto-generated by Fern from our API Definition.

import typing

CustomDomainCustomClientIpHeaderEnum = typing.Union[
    typing.Literal["true-client-ip", "cf-connecting-ip", "x-forwarded-for", "x-azure-clientip", ""], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .create_client_authentication_method_self_signed_tls_client_auth_credentials import (
    CreateClientAuthenticationMethodSelfSignedTlsClientAuthCredentials,
)


class CreateClientAuthenticationMethodSelfSignedTlsClientAuth(UniversalBaseModel):
    """
    Defines `self_signed_tls_client_auth` client authentication method. If the property is defined, the client is configured to use mTLS authentication method utilizing self-signed certificate.
    """

    credentials: CreateClientAuthenticationMethodSelfSignedTlsClientAuthCredentials

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



"""
Pytest plugin that manages the WireMock container lifecycle for wire tests.

This plugin is loaded globally for the test suite and is responsible for
starting and stopping the WireMock container exactly once per test run,
including when running with pytest-xdist over the entire project.

It lives under tests/ (as tests/conftest.py) and is discovered automatically
by pytest's normal test collection rules.
"""

import os
import subprocess

import pytest

_STARTED: bool = False
_EXTERNAL: bool = False  # True when using an external WireMock instance (skip container lifecycle)
_WIREMOCK_URL: str = "http://localhost:8080"  # Default, will be updated after container starts
_PROJECT_NAME: str = "auth0-api"

# This file lives at tests/conftest.py, so the project root is one level up.
_PROJECT_ROOT = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
_COMPOSE_FILE = os.path.join(_PROJECT_ROOT, "wiremock", "docker-compose.test.yml")


def _get_wiremock_port() -> str:
    """Gets the dynamically assigned port for the WireMock container."""
    try:
        result = subprocess.run(
            ["docker", "compose", "-f", _COMPOSE_FILE, "-p", _PROJECT_NAME, "port", "wiremock", "8080"],
            check=True,
            capture_output=True,
            text=True,
        )
        # Output is like "0.0.0.0:32768" or "[::]:32768"
        port = result.stdout.strip().split(":")[-1]
        return port
    except subprocess.CalledProcessError:
        return "8080"  # Fallback to default


def _start_wiremock() -> None:
    """Starts the WireMock container using docker-compose."""
    global _STARTED, _EXTERNAL, _WIREMOCK_URL
    if _STARTED:
        return

    # If WIREMOCK_URL is already set (e.g., by CI/CD pipeline), skip container management
    existing_url = os.environ.get("WIREMOCK_URL")
    if existing_url:
        _WIREMOCK_URL = existing_url
        _EXTERNAL = True
        _STARTED = True
        print(f"\nUsing external WireMock at {_WIREMOCK_URL} (container management skipped)")
        return

    print(f"\nStarting WireMock container (project: {_PROJECT_NAME})...")
    try:
        subprocess.run(
            ["docker", "compose", "-f", _COMPOSE_FILE, "-p", _PROJECT_NAME, "up", "-d", "--wait"],
            check=True,
            capture_output=True,
            text=True,
        )
        _WIREMOCK_PORT = _get_wiremock_port()
        _WIREMOCK_URL = f"http://localhost:{_WIREMOCK_PORT}"
        os.environ["WIREMOCK_URL"] = _WIREMOCK_URL
        print(f"WireMock container is ready at {_WIREMOCK_URL}")
        _STARTED = True
    except subprocess.CalledProcessError as e:
        print(f"Failed to start WireMock: {e.stderr}")
        raise


def _stop_wiremock() -> None:
    """Stops and removes the WireMock container."""
    if _EXTERNAL:
        # Container is managed externally; nothing to tear down.
        return

    print("\nStopping WireMock container...")
    subprocess.run(
        ["docker", "compose", "-f", _COMPOSE_FILE, "-p", _PROJECT_NAME, "down", "-v"],
        check=False,
        capture_output=True,
    )


def _is_xdist_worker(config: pytest.Config) -> bool:
    """
    Determines if the current process is an xdist worker.

    In pytest-xdist, worker processes have a 'workerinput' attribute
    on the config object, while the controller process does not.
    """
    return hasattr(config, "workerinput")


def _has_httpx_aiohttp() -> bool:
    """Check if httpx_aiohttp is importable."""
    try:
        import httpx_aiohttp  # type: ignore[import-not-found]  # noqa: F401

        return True
    except ImportError:
        return False


def pytest_collection_modifyitems(config: pytest.Config, items: list) -> None:
    """Auto-skip @pytest.mark.aiohttp tests when httpx_aiohttp is not installed."""
    if _has_httpx_aiohttp():
        return
    skip_aiohttp = pytest.mark.skip(reason="httpx_aiohttp not installed")
    for item in items:
        if "aiohttp" in item.keywords:
            item.add_marker(skip_aiohttp)


def pytest_configure(config: pytest.Config) -> None:
    """
    Pytest hook that runs during test session setup.

    Starts WireMock container only from the controller process (xdist)
    or the single process (non-xdist). This ensures only one container
    is started regardless of the number of worker processes.
    """
    if _is_xdist_worker(config):
        # Workers never manage the container lifecycle.
        return

    _start_wiremock()


def pytest_unconfigure(config: pytest.Config) -> None:
    """
    Pytest hook that runs during test session teardown.

    Stops WireMock container only from the controller process (xdist)
    or the single process (non-xdist). This ensures the container is
    cleaned up after all workers have finished.
    """
    if _is_xdist_worker(config):
        # Workers never manage the container lifecycle.
        return

    _stop_wiremock()



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .action_binding_type_enum import ActionBindingTypeEnum
from .action_trigger_compatible_trigger import ActionTriggerCompatibleTrigger
from .action_trigger_type_enum import ActionTriggerTypeEnum


class ActionTrigger(UniversalBaseModel):
    id: ActionTriggerTypeEnum
    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    The version of a trigger. v1, v2, etc.
    """

    status: typing.Optional[str] = pydantic.Field(default=None)
    """
    status points to the trigger status.
    """

    runtimes: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    runtimes supported by this trigger.
    """

    default_runtime: typing.Optional[str] = pydantic.Field(default=None)
    """
    Runtime that will be used when none is specified when creating an action.
    """

    compatible_triggers: typing.Optional[typing.List[ActionTriggerCompatibleTrigger]] = pydantic.Field(default=None)
    """
    compatible_triggers informs which other trigger supports the same event and api.
    """

    binding_policy: typing.Optional[ActionBindingTypeEnum] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .client_token_exchange_type_enum import ClientTokenExchangeTypeEnum


class ClientTokenExchangeConfiguration(UniversalBaseModel):
    """
    Configuration for token exchange.
    """

    allow_any_profile_of_type: typing.Optional[typing.List[ClientTokenExchangeTypeEnum]] = pydantic.Field(default=None)
    """
    List the enabled token exchange types for this client.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .connection_digest_algorithm_enum_saml import ConnectionDigestAlgorithmEnumSaml

ConnectionDigestAlgorithmSaml = ConnectionDigestAlgorithmEnumSaml
"""
Algorithm used for computing digest values when signing SAML requests and logout requests. Defaults to 'sha256'.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .form_field_custom_config_params import FormFieldCustomConfigParams
from .form_field_custom_config_schema import FormFieldCustomConfigSchema


class FormFieldCustomConfig(UniversalBaseModel):
    schema_: typing_extensions.Annotated[
        FormFieldCustomConfigSchema, FieldMetadata(alias="schema"), pydantic.Field(alias="schema")
    ]
    code: str
    css: typing.Optional[str] = None
    params: typing.Optional[FormFieldCustomConfigParams] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .form_ending_node_after_submit import FormEndingNodeAfterSubmit
from .form_ending_node_redirection import FormEndingNodeRedirection
from .form_ending_node_resume_flow_true_const import FormEndingNodeResumeFlowTrueConst
from .form_node_coordinates import FormNodeCoordinates


class FormEndingNode(UniversalBaseModel):
    redirection: typing.Optional[FormEndingNodeRedirection] = None
    after_submit: typing.Optional[FormEndingNodeAfterSubmit] = None
    coordinates: typing.Optional[FormNodeCoordinates] = None
    resume_flow: typing.Optional[FormEndingNodeResumeFlowTrueConst] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawKeysClient, RawKeysClient

if typing.TYPE_CHECKING:
    from .custom_signing.client import AsyncCustomSigningClient, CustomSigningClient
    from .encryption.client import AsyncEncryptionClient, EncryptionClient
    from .signing.client import AsyncSigningClient, SigningClient


class KeysClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawKeysClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._custom_signing: typing.Optional[CustomSigningClient] = None
        self._encryption: typing.Optional[EncryptionClient] = None
        self._signing: typing.Optional[SigningClient] = None

    @property
    def with_raw_response(self) -> RawKeysClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawKeysClient
        """
        return self._raw_client

    @property
    def custom_signing(self):
        if self._custom_signing is None:
            from .custom_signing.client import CustomSigningClient  # noqa: E402

            self._custom_signing = CustomSigningClient(client_wrapper=self._client_wrapper)
        return self._custom_signing

    @property
    def encryption(self):
        if self._encryption is None:
            from .encryption.client import EncryptionClient  # noqa: E402

            self._encryption = EncryptionClient(client_wrapper=self._client_wrapper)
        return self._encryption

    @property
    def signing(self):
        if self._signing is None:
            from .signing.client import SigningClient  # noqa: E402

            self._signing = SigningClient(client_wrapper=self._client_wrapper)
        return self._signing


class AsyncKeysClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawKeysClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._custom_signing: typing.Optional[AsyncCustomSigningClient] = None
        self._encryption: typing.Optional[AsyncEncryptionClient] = None
        self._signing: typing.Optional[AsyncSigningClient] = None

    @property
    def with_raw_response(self) -> AsyncRawKeysClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawKeysClient
        """
        return self._raw_client

    @property
    def custom_signing(self):
        if self._custom_signing is None:
            from .custom_signing.client import AsyncCustomSigningClient  # noqa: E402

            self._custom_signing = AsyncCustomSigningClient(client_wrapper=self._client_wrapper)
        return self._custom_signing

    @property
    def encryption(self):
        if self._encryption is None:
            from .encryption.client import AsyncEncryptionClient  # noqa: E402

            self._encryption = AsyncEncryptionClient(client_wrapper=self._client_wrapper)
        return self._encryption

    @property
    def signing(self):
        if self._signing is None:
            from .signing.client import AsyncSigningClient  # noqa: E402

            self._signing = AsyncSigningClient(client_wrapper=self._client_wrapper)
        return self._signing



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .client_default_organization_flows_enum import ClientDefaultOrganizationFlowsEnum


class ClientDefaultOrganization(UniversalBaseModel):
    """
    Defines the default Organization ID and flows
    """

    organization_id: str = pydantic.Field()
    """
    The default Organization ID to be used
    """

    flows: typing.List[ClientDefaultOrganizationFlowsEnum] = pydantic.Field()
    """
    The default Organization usage
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.create_hook_secret_request_content import CreateHookSecretRequestContent
from ...types.delete_hook_secret_request_content import DeleteHookSecretRequestContent
from ...types.get_hook_secret_response_content import GetHookSecretResponseContent
from ...types.update_hook_secret_request_content import UpdateHookSecretRequestContent
from .raw_client import AsyncRawSecretsClient, RawSecretsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SecretsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSecretsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSecretsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSecretsClient
        """
        return self._raw_client

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetHookSecretResponseContent:
        """
        Retrieve a hook's secrets by the ID of the hook.

        Parameters
        ----------
        id : str
            ID of the hook to retrieve secrets from.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetHookSecretResponseContent
            Hook secrets successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.hooks.secrets.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def create(
        self,
        id: str,
        *,
        request: CreateHookSecretRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Add one or more secrets to an existing hook. Accepts an object of key-value pairs, where the key is the name of the secret. A hook can have a maximum of 20 secrets.

        Parameters
        ----------
        id : str
            The id of the hook to retrieve

        request : CreateHookSecretRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.hooks.secrets.create(
            id="id",
            request={"key": "value"},
        )
        """
        _response = self._raw_client.create(id, request=request, request_options=request_options)
        return _response.data

    def delete(
        self,
        id: str,
        *,
        request: DeleteHookSecretRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Delete one or more existing secrets for a given hook. Accepts an array of secret names to delete.

        Parameters
        ----------
        id : str
            ID of the hook whose secrets to delete.

        request : DeleteHookSecretRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.hooks.secrets.delete(
            id="id",
            request=["string"],
        )
        """
        _response = self._raw_client.delete(id, request=request, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        request: UpdateHookSecretRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Update one or more existing secrets for an existing hook. Accepts an object of key-value pairs, where the key is the name of the existing secret.

        Parameters
        ----------
        id : str
            ID of the hook whose secrets to update.

        request : UpdateHookSecretRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.hooks.secrets.update(
            id="id",
            request={"key": "value"},
        )
        """
        _response = self._raw_client.update(id, request=request, request_options=request_options)
        return _response.data


class AsyncSecretsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSecretsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSecretsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSecretsClient
        """
        return self._raw_client

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetHookSecretResponseContent:
        """
        Retrieve a hook's secrets by the ID of the hook.

        Parameters
        ----------
        id : str
            ID of the hook to retrieve secrets from.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetHookSecretResponseContent
            Hook secrets successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.hooks.secrets.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def create(
        self,
        id: str,
        *,
        request: CreateHookSecretRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Add one or more secrets to an existing hook. Accepts an object of key-value pairs, where the key is the name of the secret. A hook can have a maximum of 20 secrets.

        Parameters
        ----------
        id : str
            The id of the hook to retrieve

        request : CreateHookSecretRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.hooks.secrets.create(
                id="id",
                request={"key": "value"},
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(id, request=request, request_options=request_options)
        return _response.data

    async def delete(
        self,
        id: str,
        *,
        request: DeleteHookSecretRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Delete one or more existing secrets for a given hook. Accepts an array of secret names to delete.

        Parameters
        ----------
        id : str
            ID of the hook whose secrets to delete.

        request : DeleteHookSecretRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.hooks.secrets.delete(
                id="id",
                request=["string"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request=request, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        request: UpdateHookSecretRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Update one or more existing secrets for an existing hook. Accepts an object of key-value pairs, where the key is the name of the existing secret.

        Parameters
        ----------
        id : str
            ID of the hook whose secrets to update.

        request : UpdateHookSecretRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.hooks.secrets.update(
                id="id",
                request={"key": "value"},
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(id, request=request, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from .update_default_canonical_domain_response_content import UpdateDefaultCanonicalDomainResponseContent
from .update_default_custom_domain_response_content import UpdateDefaultCustomDomainResponseContent

UpdateDefaultDomainResponseContent = typing.Union[
    UpdateDefaultCustomDomainResponseContent, UpdateDefaultCanonicalDomainResponseContent
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class UserBlockIdentifier(UniversalBaseModel):
    identifier: typing.Optional[str] = pydantic.Field(default=None)
    """
    Identifier (should be any of an `email`, `username`, or `phone_number`)
    """

    ip: typing.Optional[str] = pydantic.Field(default=None)
    """
    IP Address
    """

    connection: typing.Optional[str] = pydantic.Field(default=None)
    """
    Connection identifier
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamCloudEventConnectionDeletedObject1OptionsTypeEnum = typing.Union[typing.Literal["back_channel"], typing.Any]



import unittest
from unittest import mock

from auth0.authentication.enterprise import Enterprise


class TestEnterprise(unittest.TestCase):
    @mock.patch("auth0.authentication.enterprise.Enterprise.get")
    def test_saml_metadata(self, mock_get):
        e = Enterprise("my.domain.com", "cid")

        e.saml_metadata()

        mock_get.assert_called_with(url="https://my.domain.com/samlp/metadata/cid")

    @mock.patch("auth0.authentication.enterprise.Enterprise.get")
    def test_wsfed_metadata(self, mock_get):
        e = Enterprise("my.domain.com", "cid")

        e.wsfed_metadata()

        mock_get.assert_called_with(
            url=(
                "https://my.domain.com/wsfed/FederationMetadata"
                "/2007-06/FederationMetadata.xml"
            )
        )



from __future__ import annotations

from typing import Any

from .base import AuthenticationBase


class Database(AuthenticationBase):
    """Database & Active Directory / LDAP Authentication.

    Args:
        domain (str): Your auth0 domain (e.g: username.auth0.com)
    """

    def signup(
        self,
        email: str,
        password: str,
        connection: str,
        username: str | None = None,
        user_metadata: dict[str, Any] | None = None,
        given_name: str | None = None,
        family_name: str | None = None,
        name: str | None = None,
        nickname: str | None = None,
        picture: str | None = None,
    ) -> dict[str, Any]:
        """Signup using email and password.

        Args:
           email (str): The user's email address.

           password (str): The user's desired password.

           connection (str): The name of the database connection where this user should be created.

           username (str, optional): The user's username, if required by the database connection.

           user_metadata (dict, optional): Additional key-value information to store for the user.
                    Some limitations apply, see: https://auth0.com/docs/metadata#metadata-restrictions

           given_name (str, optional): The user's given name(s).

           family_name (str, optional): The user's family name(s).

           name (str, optional): The user's full name.

           nickname (str, optional): The user's nickname.

           picture (str, optional): A URI pointing to the user's picture.


        See: https://auth0.com/docs/api/authentication#signup
        """
        body: dict[str, Any] = {
            "client_id": self.client_id,
            "email": email,
            "password": password,
            "connection": connection,
        }
        if username:
            body.update({"username": username})
        if user_metadata:
            body.update({"user_metadata": user_metadata})
        if given_name:
            body.update({"given_name": given_name})
        if family_name:
            body.update({"family_name": family_name})
        if name:
            body.update({"name": name})
        if nickname:
            body.update({"nickname": nickname})
        if picture:
            body.update({"picture": picture})

        data: dict[str, Any] = self.post(
            f"{self.protocol}://{self.domain}/dbconnections/signup", data=body
        )
        return data

    def change_password(
        self,
        email: str,
        connection: str,
        password: str | None = None,
        organization: str | None = None,
    ) -> str:
        """Asks to change a password for a given user.

        email (str): The user's email address.

        connection (str): The name of the database connection where this user should be created.

        organization (str, optional): The id of the Organization associated with the user.
        """
        body = {
            "client_id": self.client_id,
            "email": email,
            "connection": connection,
        }
        if organization:
            body["organization"] = organization

        data: str = self.post(
            f"{self.protocol}://{self.domain}/dbconnections/change_password",
            data=body,
        )
        return data



from __future__ import annotations

from typing import Any


class Auth0Error(Exception):
    def __init__(
        self,
        status_code: int,
        error_code: str,
        message: str,
        content: Any | None = None,
        headers: Any | None = None,
    ) -> None:
        self.status_code = status_code
        self.error_code = error_code
        self.message = message
        self.content = content
        self.headers = headers

    def __str__(self) -> str:
        return f"{self.status_code}: {self.message}"


class RateLimitError(Auth0Error):
    def __init__(self, error_code: str, message: str, reset_at: int, headers: Any | None = None) -> None:
        super().__init__(status_code=429, error_code=error_code, message=message, headers=headers)
        self.reset_at = reset_at


class TokenValidationError(Exception):
    pass



# This file was auto-generated by Fern from our API Definition.

import typing

SelfServiceProfileSsoTicketProvisioningScopeEnum = typing.Union[
    typing.Literal[
        "get:users",
        "post:users",
        "put:users",
        "patch:users",
        "delete:users",
        "get:groups",
        "post:groups",
        "put:groups",
        "patch:groups",
        "delete:groups",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .acul_configs_item import AculConfigsItem

AculConfigs = typing.List[AculConfigsItem]
"""
Array of screen configurations to update
"""
